@@ -25,7 +25,36 @@ def service(x, param, url, step_param_names, step_objfunc, calib_params, files=N
     return cost
 
 
-def global_best(steps, rounds, args, n_particles, iters, options, ctol=1.0E-6, ftol=-np.inf):
+def global_best(steps, rounds, args, n_particles, iters, options, rtol=1.0E-6, ftol=-np.inf):
+    """Performs stepwise PSO.
+
+        Performs a stepwise particle swarm optimization
+
+        Parameters
+        ----------
+        steps : dict
+            step definitions
+        rounds : tuple
+            round definition,  (min,max)
+        args : dict
+            static service args
+        n_particles : int
+            number of particles
+        iters : int
+            number of iterations
+        options : dict
+            PSO options (see pyswarms)
+        rtol : float
+            tolerance to exit rounds (default: 1.0E-6)
+        ftol : float
+            PSO tolerance
+
+        Returns
+        -------
+        tuple
+            optimizer, best,  all PSO optimizer, the global best.
+    """
+
     utils.check_url(args['url'])
 
     min_rounds = 1
@@ -58,14 +87,13 @@ def global_best(steps, rounds, args, n_particles, iters, options, ctol=1.0E-6, f
                 optimizer[s] = GlobalBestPSO(n_particles, len(param_names),
                                              options=options, bounds=bounds, ftol=ftol)
 
-            print('\n>>>>> R{}/S{}  particle params: {}  using: {}\n'.format(r + 1, s + 1, param_names,
+            print('\n>>>>> R{}/S{}  particle params: {}  calibrated params: {}\n'.format(r + 1, s + 1, param_names,
                                                                              args['calib_params']))
 
             cost, pos = optimizer[s].optimize(service, iters=iters, **args)
 
             # capture the best cost
-            # if cost < best_cost[s]:
-            if np.abs(cost - best_cost[s]) > ctol:
+            if cost < best_cost[s] and np.abs(cost - best_cost[s]) > rtol:
                 best_cost[s] = cost
                 no_improvement[s] = False
                 utils.annotate_step(best_cost[s], pos, steps, s)
@@ -74,11 +102,12 @@ def global_best(steps, rounds, args, n_particles, iters, options, ctol=1.0E-6, f
 
             # print(json.dumps(steps, sort_keys=False, indent=2))
 
+        round_cost = np.sum(best_cost)
         # if no improvement in all steps, break out of rounds prematurely
-        print('\n  Round summary, bestcost: {}, improvement:{} '.format(best_cost, np.invert(no_improvement)))
+        print('\n  Round summary, round_cost:{}, step_costs: {}, improvement:{} '.format(round_cost, best_cost, np.invert(no_improvement)))
         # but start checking only after min_rounds
         if (r + 1 >= min_rounds) and all(no_improvement):
             break
 
-    print('Done after {}/{} rounds'.format(r + 1, max_rounds))
+    print('Done after {} out of {} rounds'.format(r + 1, max_rounds))
     return optimizer, cost  # return the last output
